using Microsoft.Win32;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Net;
using System.Net.NetworkInformation;
using System.Text;
using System.Threading;
using System.Timers;
using System.Windows.Forms;

namespace LawnLogin {
	public struct Settings {
		public string account;
		public bool autoStart;					// start LawnLogin with Windows
		public string password;
		public bool disableISS;
		public ConnectAction connectAction;
		public bool disableNotify;
		public UpdateFrequency updateFrequency;
		public DateTime lastUpdate;
	}

	public enum ConnectAction {
		FullLogin = 0,
		ShowDialog = 1,
		DoNothing = 2
	}

	public enum ConnectivityStatus {
		LawnLoggedIn,
		LawnLoggedOut,
		OutsideLawn,
		Unknown
	}

	public enum UpdateFrequency {
		Never = 0,
		Daily = 1,
		Weekly = 2,
		Monthly = 3
	}

	static class Program {

		/* --- Configuration variables --- */

		static string lawnUrl = "http://www.lawn.gatech.edu/";										// URL to visit when the user chooses "Visit LAWN Website"
		static string loginFormUrl = "https://auth.lawn.gatech.edu/index.php";						// URL of the login form that accepts our POST request
		static string logoutUrl = "https://auth.lawn.gatech.edu/logout.php";						// URL of the logout page
		static string statusUrl = "https://auth.lawn.gatech.edu/login_status.php?output=text";		// URL of the status request page
		static string updateUrl = "http://www.hydrous.net/version/lawnlogin";						// URL to query for current version information

		static string regKey = @"HKEY_CURRENT_USER\Software\Hydrous\LawnLogin";						// Include the entire registry path
		static string regKeyRun = @"Software\Microsoft\Windows\CurrentVersion\Run";					// Do not include the HKCU part

		static int webTimeout = 8000;		// number of ms to wait before deciding that a web request has failed

		/* --- Globals --- */

		static Settings currentSettings;
		static ConnectivityStatus lastStatus = ConnectivityStatus.Unknown;
		static bool forcedCheck = false;

		static System.Timers.Timer lawnStatusTimer;
		static System.Timers.Timer updateTimer;
		static Thread reactThread;
		static Thread updateThread;
		static AutoResetEvent reactEvent;

		/* --- Notification icon objects --- */

		static ContextMenu mnuNotify = new ContextMenu();
		static MenuItem mnuOptions = new MenuItem(Properties.Resources.MenuOptions);
		static MenuItem mnuLogin = new MenuItem(Properties.Resources.MenuLogin);
		static MenuItem mnuLogout = new MenuItem(Properties.Resources.MenuLogout);
		static MenuItem mnuAdvanced = new MenuItem(Properties.Resources.MenuAdvanced);
		static MenuItem mnuCheckNow = new MenuItem(Properties.Resources.MenuCheckNow);
		static MenuItem mnuForceLogin = new MenuItem(Properties.Resources.MenuForceLogin);
		static MenuItem mnuForceLogout = new MenuItem(Properties.Resources.MenuForceLogout);
		static MenuItem mnuSep1 = new MenuItem("-");
		static MenuItem mnuCheckUpdates = new MenuItem(Properties.Resources.MenuCheckUpdates);
		static MenuItem mnuSep2 = new MenuItem("-");
		static MenuItem mnuVisitWebsite = new MenuItem(Properties.Resources.MenuVisitWebsite);
		static MenuItem mnuAbout = new MenuItem(Properties.Resources.MenuAbout);
		static MenuItem mnuSep3 = new MenuItem("-");
		static MenuItem mnuExit = new MenuItem(Properties.Resources.MenuExit);
		static NotifyIcon icoNotify = new NotifyIcon();

		/* --- Methods --- */

		/**
		 * Determine if this application EXE is already running
		 * 
		 * @return True if the same EXE is already loaded, false otherwise
		 */
		static bool AlreadyRunning() {
			Process currProc = Process.GetCurrentProcess();
			Process[] procs = Process.GetProcessesByName(currProc.ProcessName);
			foreach (Process thisProc in procs) {
				if ((thisProc.Id != currProc.Id) && (thisProc.MainModule.FileName == currProc.MainModule.FileName))
					return true;
			}
			return false;
		}

		/**
		 * Start the update check worker
		 */
		static void CheckUpdates() {
			if (updateThread != null) {
				updateThread.Abort();
				updateThread = null;
			}
			updateThread = new Thread(new ThreadStart(UpdateCheckWorker));
			updateThread.Start();
		}

		/**
		 * Decrypt a password stored in the registry
		 * 
		 * @param text Encrypted string
		 * @return Decrypted password
		 */
		static string Decrypt(string text) {
			// Deobfuscate passwords
			ASCIIEncoding encoding = new ASCIIEncoding();
			byte[] btext = Convert.FromBase64String(text);
			return encoding.GetString(btext);
		}

		/**
		 * Disable the options menu within the notification icon's context menu
		 */
		static public void DisableOptionsMenu() {
			mnuOptions.Enabled = false;
		}

		/**
		 * Enable the options menu within the notification icon's context menu
		 */
		static public void EnableOptionsMenu() {
			mnuOptions.Enabled = true;
		}

		/**
		 * Decrypt a password stored in the registry. This is just a simple form of base64
		 * obfuscation to prevent the simple visual exposure of a user's stored password. It's
		 * certainly not a secure form of encryption.
		 * 
		 * @param text Decrypted password
		 * @return Encrypted string
		 */
		static string Encrypt(string text) {
			ASCIIEncoding encoding = new ASCIIEncoding();
			byte[] btext = encoding.GetBytes(text);
			return Convert.ToBase64String(btext);
		}

		/**
		 * Login to LAWN with the specified details
		 * 
		 * @param account User's GT account name
		 * @param password User's GT account password
		 * @param disableISS Inform LAWN to disable firewall protection on incoming ports
		 * @return True if successful, false if not (due to bad password, LAWN not available, etc.)
		 */
		static public bool Login(string account, string password, bool disableISS) {
			WebResponse response;
			Stream stream;
			StreamReader reader;
			string contents;

			// Set up the POST data
			account = Uri.EscapeDataString(account);
			password = Uri.EscapeDataString(password);
			String toSend = "username=" + account + "&password=" + password + (disableISS ? "&iss=false" : "") + "&output=text";
			ASCIIEncoding encoding = new ASCIIEncoding();
			byte[] postdata = encoding.GetBytes(toSend);

			// Set up the web request
			HttpWebRequest web = (HttpWebRequest)WebRequest.Create(loginFormUrl);
			web.Method = "POST";
			web.ContentType = "application/x-www-form-urlencoded";
			web.ContentLength = postdata.Length;
			web.Timeout = webTimeout;
			try {
				Stream webStream = web.GetRequestStream();
				webStream.Write(postdata, 0, postdata.Length);		// send POST data
			} catch (Exception) {
				Notify(Properties.Resources.LoginErrorConnect, ToolTipIcon.Error);
				return false;
			}

			// If POST data was sent successfully ...
			try {
				response = web.GetResponse();
				stream = response.GetResponseStream();
				reader = new StreamReader(stream, Encoding.UTF8);
				contents = reader.ReadToEnd();
			} catch (Exception) {
				Notify(Properties.Resources.LoginErrorUnusableResponse, ToolTipIcon.Error);
				return false;
			}

			// Check for login success
			if (contents.StartsWith("success") || contents.Contains("Logging you into LAWN")) {		// Please Tech ... fix this.
				// Get the state set up
				mnuLogin.Visible = false;
				mnuLogout.Visible = true;
				icoNotify.Text = Properties.Resources.StatusLoggedIn;
				icoNotify.Icon = Properties.Resources.LoggedIn;
				lastStatus = ConnectivityStatus.LawnLoggedIn;
				return true;
			}

			// If there's a failure, report the error message
			Notify(contents.Trim(), ToolTipIcon.Error);
			return false;
		}

		/**
		 * Login to LAWN and possibly remember the user's account information as the new default
		 * 
		 * @param account User's GT account name
		 * @param password User's GT account password
		 * @param disableISS Inform LAWN to disable firewall protection on incoming ports
		 * @param remember If true, remember these details as the new default
		 */
		static public bool Login(string account, string password, bool disableISS, bool remember) {
			if (remember) {
				Settings newSettings = currentSettings;
				newSettings.account = account;
				newSettings.password = password;
				newSettings.disableISS = disableISS;
				UpdateSettings(newSettings);
			}

			return Login(account, password, disableISS);
		}

		/**
		 * Logout of LAWN
		 */
		static public bool Logout() {
			WebResponse response;
			Stream stream;
			StreamReader reader;
			string contents;

			// Set up the web request
			HttpWebRequest web = (HttpWebRequest)WebRequest.Create(logoutUrl);
			web.Timeout = webTimeout;
			try {
				response = web.GetResponse();
				stream = response.GetResponseStream();
				reader = new StreamReader(stream, Encoding.UTF8);
				contents = reader.ReadToEnd();
			} catch (Exception) {
				Notify(Properties.Resources.LogoutErrorUnusableResponse, ToolTipIcon.Error);
				return false;
			}

			StateReact();		// force a query to LAWN to determine whether we're now logged in or not
			return true;
		}

		/**
		 * Determine our current status on LAWN. If there is a problem fetching the page, or the
		 * page contains unexpected text, this method will return Unknown.
		 * 
		 * @return An enum representing the current status
		 */
		static ConnectivityStatus GetLawnStatus() {
			WebResponse response;
			Stream stream;
			StreamReader reader;
			string contents;

			// Ask LAWN if we're on LAWN and/or logged in
			HttpWebRequest web = (HttpWebRequest)WebRequest.Create(statusUrl);
			web.Timeout = webTimeout;
			try {
				response = web.GetResponse();
				stream = response.GetResponseStream();
				reader = new StreamReader(stream, Encoding.UTF8);
				contents = reader.ReadToEnd().Trim();
			} catch (Exception) {
				return ConnectivityStatus.Unknown;
			}

			if (contents.StartsWith("not_on_LAWN")) return ConnectivityStatus.OutsideLawn;
            if (contents.StartsWith("logged_in")) return ConnectivityStatus.LawnLoggedIn;
			if (contents.StartsWith("not_authenticated")) return ConnectivityStatus.LawnLoggedOut;

			return ConnectivityStatus.Unknown;
		}

		/**
		 * Change the notification area text and icon
		 * 
		 * @param text New text tooltip to apply
		 * @param icon Icon object to apply
		 */
		static public void Notify(string text, ToolTipIcon icon) {
			icoNotify.ShowBalloonTip(5000, "LawnLogin", text, icon);
		}

		/**
		 * After notifying the user of an available update, reset the update clock to the current
		 * time.
		 */
		static void ResetUpdateClock() {
			currentSettings.lastUpdate = DateTime.Now;
			Registry.SetValue(regKey, "LastUpdate", currentSettings.lastUpdate.ToString(), RegistryValueKind.String);
		}

		/**
		 * Build the menu items within the notification icon's context menu
		 */
		static void SetupNotifyIcon() {
			// Menu options
			mnuOptions.DefaultItem = true;
			mnuLogout.Visible = false;

			// Register event handlers
			mnuOptions.Click += new System.EventHandler(mnuOptions_Click);
			mnuLogin.Click += new System.EventHandler(mnuLogin_Click);
			mnuLogout.Click += new System.EventHandler(mnuLogout_Click);
			mnuCheckNow.Click += new System.EventHandler(mnuCheckNow_Click);
			mnuForceLogin.Click += new System.EventHandler(mnuForceLogin_Click);
			mnuForceLogout.Click += new System.EventHandler(mnuForceLogout_Click);
			mnuCheckUpdates.Click += new System.EventHandler(mnuCheckUpdates_Click);
			mnuVisitWebsite.Click += new System.EventHandler(mnuVisitWebsite_Click);
			mnuAbout.Click += new System.EventHandler(mnuAbout_Click);
			mnuExit.Click += new System.EventHandler(mnuExit_Click);

			// Glue menu together
			mnuNotify.MenuItems.Add(mnuOptions);
			mnuNotify.MenuItems.Add(mnuLogin);
			mnuNotify.MenuItems.Add(mnuLogout);
			mnuNotify.MenuItems.Add(mnuAdvanced);
			mnuAdvanced.MenuItems.Add(mnuCheckNow);
			mnuAdvanced.MenuItems.Add(mnuForceLogin);
			mnuAdvanced.MenuItems.Add(mnuForceLogout);
			mnuAdvanced.MenuItems.Add(mnuSep1);
			mnuAdvanced.MenuItems.Add(mnuCheckUpdates);
			mnuNotify.MenuItems.Add(mnuSep2);
			mnuNotify.MenuItems.Add(mnuVisitWebsite);
			mnuNotify.MenuItems.Add(mnuAbout);
			mnuNotify.MenuItems.Add(mnuSep3);
			mnuNotify.MenuItems.Add(mnuExit);

			// Set up the notification icon
			icoNotify.Text = "LawnLogin";
			icoNotify.Icon = Properties.Resources.NotLawn;
			icoNotify.ContextMenu = mnuNotify;
			icoNotify.DoubleClick += new System.EventHandler(icoNotify_DoubleClick);
			icoNotify.Visible = true;
		}

		/**
		 * Determine whether we should check for a software update, based on the user's preferences
		 * and the elapsed time since the last check.
		 * 
		 * @return True if an update is appropriate, false otherwise
		 */
		static bool ShouldUpdate() {
			DateTime updateTime = DateTime.MaxValue;
			switch (currentSettings.updateFrequency) {
				case UpdateFrequency.Daily:
					updateTime = currentSettings.lastUpdate.AddDays(1);
					break;
				case UpdateFrequency.Weekly:
					updateTime = currentSettings.lastUpdate.AddDays(7);
					break;
				case UpdateFrequency.Monthly:
					updateTime = currentSettings.lastUpdate.AddMonths(1);
					break;
				default:
					return false;
			}

			return (updateTime < DateTime.Now);
		}

		/**
		 * Start the react timer, resetting it if it was already enabled
		 */
		static public void StartLawnStatusTimer() {
			lawnStatusTimer.Enabled = false;
			lawnStatusTimer.Enabled = true;
		}

		/**
		 * Start the update check timer, resetting it if it was already enabled
		 */
		static public void StartUpdateTimer() {
			updateTimer.Enabled = false;
			updateTimer.Enabled = true;
		}

		/**
		 * Inform the worker thread that it should react to the state whenever possible. This
		 * means that it should check for LAWN status, log in, check for updates, etc. whenever it
		 * becomes idle.
		 */
		static void StateReact() {
			reactEvent.Set();
		}

		/**
		 * Cancel the pending react timer, if any
		 */
		static public void StopLawnStatusTimer() {
			lawnStatusTimer.Enabled = false;
		}

		/**
		 * Cancel the pending update check timer, if any
		 */
		static public void StopUpdateTimer() {
			updateTimer.Enabled = false;
		}

		/**
		 * Save a new group of settings in memory and to the registry
		 * 
		 * @param newSettings The new settings object to apply
		 */
		static public void UpdateSettings(Settings newSettings) {
			currentSettings = newSettings;

			// Save the new settings to the registry
			Registry.SetValue(regKey, "Account", currentSettings.account, RegistryValueKind.String);
			Registry.SetValue(regKey, "Password", Encrypt(currentSettings.password), RegistryValueKind.String);
			Registry.SetValue(regKey, "Action", currentSettings.connectAction, RegistryValueKind.DWord);
			Registry.SetValue(regKey, "DisableISS", (currentSettings.disableISS ? 1 : 0), RegistryValueKind.DWord);
			Registry.SetValue(regKey, "DisableNotify", (currentSettings.disableNotify ? 1 : 0), RegistryValueKind.DWord);
			Registry.SetValue(regKey, "UpdateFrequency", currentSettings.updateFrequency, RegistryValueKind.DWord);

			// Windows startup setting
			RegistryKey runKey = Registry.CurrentUser.OpenSubKey(regKeyRun, true);
			if (currentSettings.autoStart) {
				runKey.SetValue("LawnLogin", Application.ExecutablePath, RegistryValueKind.String);
			} else {
				try {
					runKey.DeleteValue("LawnLogin");
				} catch (Exception) {
					// If the value does not already exist, simply ignore it.
				}
			}
		}

		/* --- Threads --- */

		/**
		 * Thread to handle state changes by watching the status of reactEvent. When reactEvent is
		 * set, the thread will check the status of LAWN as well as check for updates.
		 */
		static void StateReactWorker() {
			Form loginForm;
			ConnectivityStatus currentStatus;

			while (true) {
				reactEvent.WaitOne();					// wait for an instruction to react to state

				currentStatus = GetLawnStatus();
				StopLawnStatusTimer();							// cancel any other pending checks
				switch (currentStatus) {
					case ConnectivityStatus.LawnLoggedIn:		// we're already logged in
						mnuLogin.Visible = false;
						mnuLogout.Visible = true;
						icoNotify.Text = Properties.Resources.StatusLoggedIn;
						icoNotify.Icon = Properties.Resources.LoggedIn;
						break;
					case ConnectivityStatus.LawnLoggedOut:		// we're not logged in; react based on previous state and user preferences
						mnuLogin.Visible = true;
						mnuLogout.Visible = false;
						icoNotify.Text = Properties.Resources.StatusLoggedOut;
						icoNotify.Icon = Properties.Resources.LoggedOut;
						if (lastStatus == ConnectivityStatus.OutsideLawn || lastStatus == ConnectivityStatus.Unknown) {
							if (currentSettings.connectAction == ConnectAction.FullLogin &&
								currentSettings.account.Length > 0 &&
								currentSettings.password.Length > 0) {
								// Automatically log in now
								bool success = Login(currentSettings.account, currentSettings.password, currentSettings.disableISS);
								if (success) {
									if (!currentSettings.disableNotify) {
										Notify(Properties.Resources.LoginAutoNotify, ToolTipIcon.Info);
									}
								} else {
									loginForm = new frmLogin(currentSettings.account, currentSettings.password, currentSettings.disableISS);
									Application.Run(loginForm);
								}
							} else if (currentSettings.connectAction == ConnectAction.ShowDialog) {
								// Show the login doalog
								loginForm = new frmLogin(currentSettings.account, currentSettings.password, currentSettings.disableISS);
								Application.Run(loginForm);
							}
						}
						break;
					case ConnectivityStatus.OutsideLawn:	// LAWN is accessible but we're not on its network
						mnuLogin.Visible = false;
						mnuLogout.Visible = false;
						icoNotify.Text = Properties.Resources.StatusOutsideLawn;
						icoNotify.Icon = Properties.Resources.NotLawn;
						break;
					default:								// any other situation
						mnuLogin.Visible = false;
						mnuLogout.Visible = false;
						icoNotify.Text = Properties.Resources.StatusCannotDetermine;
						icoNotify.Icon = Properties.Resources.NotLawn;
						break;
				}

				lastStatus = currentStatus;
			}	// main thread loop
		}

		/**
		 * Thread that performs a single update (then quits)
		 */
		static void UpdateCheckWorker() {
			WebResponse response;
			Stream stream;
			StreamReader reader;
			string contents;

			// Set up the web request
			string url = updateUrl;
			if (!url.EndsWith("/"))
				url += "/";
			url += Application.ProductVersion;

			HttpWebRequest web = (HttpWebRequest)WebRequest.Create(url);
			web.Timeout = webTimeout;
			try {
				response = web.GetResponse();
				stream = response.GetResponseStream();
				reader = new StreamReader(stream, Encoding.UTF8);
				contents = reader.ReadToEnd();
			} catch (Exception) {
				if (forcedCheck) {
					MessageBox.Show("Unable to determine the current version. Is your internet connection active?", "LawnLogin", MessageBoxButtons.OK, MessageBoxIcon.Error);
					forcedCheck = false;
				}
				return;
			}
			
			if (contents.StartsWith("Current")) {
				if (forcedCheck) {
					MessageBox.Show("You are running the most recent version of LawnLogin", "LawnLogin", MessageBoxButtons.OK, MessageBoxIcon.Information);
					forcedCheck = false;
				}
				ResetUpdateClock();
				return;
			} else if (!contents.StartsWith("Old")) {
				if (forcedCheck) {
					MessageBox.Show("There was an error on the update server", "LawnLogin", MessageBoxButtons.OK, MessageBoxIcon.Error);
					forcedCheck = false;
				}
				return;
			}

			// If the contents start with "Old", we've got an update.
			string[] tabs = contents.Split('\t');
			if (tabs.Length < 4) {
				if (forcedCheck) {
					MessageBox.Show("There was an error on the update server", "LawnLogin", MessageBoxButtons.OK, MessageBoxIcon.Error);
					forcedCheck = false;
				}
				return;
			}

			forcedCheck = false;

			string version = tabs[1];
			string released = tabs[2];
			string downloadUrl = tabs[3];
			string changeLog = tabs[4];

			Form updateForm = new frmUpdate(version, released, changeLog);
			switch (updateForm.ShowDialog()) {
				case DialogResult.OK:
					System.Diagnostics.Process.Start(downloadUrl);
					break;
				case DialogResult.Cancel:
					Settings newSettings = currentSettings;
					newSettings.updateFrequency = UpdateFrequency.Never;
					UpdateSettings(newSettings);
					break;

				// Nothing to do in the case of Ignore
			}
			ResetUpdateClock();
		}

		/**
		 * The main entry point for the application.
		 */
		[STAThread]
		static void Main() {
			// Application setup
			Application.EnableVisualStyles();
			Application.SetCompatibleTextRenderingDefault(false);

			// Make sure there's only one of me
			if (AlreadyRunning()) {
				MessageBox.Show("There is already another instance of LawnLogin running", "LawnLogin", MessageBoxButtons.OK, MessageBoxIcon.Error);
				Application.Exit();
				return;
			}

			// Get program settings
			Registry.SetValue(regKey, "", "");		// sets the key's default to a dummy value in order to make sure the key exists before GetValue is called
			currentSettings = new Settings();
			currentSettings.account = (string)Registry.GetValue(regKey, "Account", "");
			currentSettings.password = Decrypt((string)Registry.GetValue(regKey, "Password", ""));
			currentSettings.connectAction = (ConnectAction)Registry.GetValue(regKey, "Action", ConnectAction.ShowDialog);
			currentSettings.disableISS = ((int)Registry.GetValue(regKey, "DisableISS", 0) == 1 ? true : false);
			currentSettings.disableNotify = ((int)Registry.GetValue(regKey, "DisableNotify", 0) == 1 ? true : false);
			currentSettings.updateFrequency = (UpdateFrequency)Registry.GetValue(regKey, "UpdateFrequency", UpdateFrequency.Weekly);
			DateTime.TryParse((string)Registry.GetValue(regKey, "LastUpdate", DateTime.MinValue.ToString()), out currentSettings.lastUpdate);

			// Windows startup setting
			try {
				RegistryKey runKey = Registry.CurrentUser.OpenSubKey(regKeyRun);
				currentSettings.autoStart = ((string)runKey.GetValue("LawnLogin", "") == "" ? false : true);
			} catch (Exception) {
				// In case there's some problem accessing the Run value
				currentSettings.autoStart = false;
			}

			// Is this the first time the program has been started?
			int hasrun = (int)Registry.GetValue(regKey, "HasRun", 0);
			if (hasrun == 0) {
				Registry.SetValue(regKey, "HasRun", 1, RegistryValueKind.DWord);
				Form optionsdialog = new frmOptions(currentSettings);
				optionsdialog.Show();
			}

			// Set up the notification icon
			SetupNotifyIcon();

			// Set up the status check timer
			lawnStatusTimer = new System.Timers.Timer(2000);		// 2 seconds
			lawnStatusTimer.AutoReset = false;
			lawnStatusTimer.Elapsed += new ElapsedEventHandler(lawnStatusTimer_Elapsed);

			// Set up the update check timer
			updateTimer = new System.Timers.Timer(300000);			// 5 minutes
			updateTimer.AutoReset = false;
			updateTimer.Elapsed += new ElapsedEventHandler(updateTimer_Elapsed);

			// Set up the status checking thread and react to the current status
			reactEvent = new AutoResetEvent(false);
			reactThread = new Thread(new ThreadStart(StateReactWorker));
			reactThread.Start();
			StateReact();

			// Begin the update timer, if appropriate
			if (ShouldUpdate())
				StartUpdateTimer();

			// Register a handler for network address change event
			NetworkChange.NetworkAddressChanged += new NetworkAddressChangedEventHandler(NetworkAddressChanged);

			// Run main message loop
			Application.Run();
		}

		/* --- Event Methods --- */

		static void icoNotify_DoubleClick(object sender, System.EventArgs e) {
			if (!mnuOptions.Enabled) return;
			MouseEventArgs me = (MouseEventArgs)e;
			if (me.Button != MouseButtons.Left) return;		// ignore right double-click
			Form optionsDialog = new frmOptions(currentSettings);
			optionsDialog.Show();
		}

		static void lawnStatusTimer_Elapsed(object sender, ElapsedEventArgs e) {
			// Implements a delay to prevent multiple quick firings of StateReact
			StateReact();
		}

		static void mnuAbout_Click(object sender, System.EventArgs e) {
			Form aboutForm = new frmAbout();
			aboutForm.Show();
		}

		static void mnuCheckNow_Click(object sender, System.EventArgs e) {
			StateReact();
		}

		static void mnuCheckUpdates_Click(object sender, System.EventArgs e) {
			forcedCheck = true;
			CheckUpdates();
		}

		static void mnuExit_Click(object sender, System.EventArgs e) {
			icoNotify.Visible = false;		// remove the notification icon
			reactThread.Abort();			// kill the react thread
			if (updateThread != null)		// kill the update thread, if there is one
				updateThread.Abort();
			Application.Exit();				// kill the application
		}

		static void mnuForceLogin_Click(object sender, System.EventArgs e) {
			Form loginForm = new frmLogin(currentSettings.account, currentSettings.password, currentSettings.disableISS);
			loginForm.Show();
		}

		static void mnuForceLogout_Click(object sender, System.EventArgs e) {
			bool success = Logout();
			if (success) Notify(Properties.Resources.LogoutNotify, ToolTipIcon.Info);
		}

		static void mnuLogin_Click(object sender, System.EventArgs e) {
			Form loginForm = new frmLogin(currentSettings.account, currentSettings.password, currentSettings.disableISS);
			loginForm.Show();
		}

		static void mnuLogout_Click(object sender, System.EventArgs e) {
			bool success = Logout();
			if (success) Notify(Properties.Resources.LogoutNotify, ToolTipIcon.Info);
		}

		static void mnuOptions_Click(object sender, System.EventArgs e) {
			Form optionsDialog = new frmOptions(currentSettings);
			optionsDialog.Show();
		}

		static void mnuVisitWebsite_Click(object sender, System.EventArgs e) {
			System.Diagnostics.Process.Start(lawnUrl);
		}

		static void NetworkAddressChanged(Object sender, EventArgs e) {
			// This event is fired if any network adapter's IP address changes
			StartLawnStatusTimer();
			if (ShouldUpdate())
				StartUpdateTimer();
		}

		static void updateTimer_Elapsed(object sender, ElapsedEventArgs e) {
			CheckUpdates();
		}
	}
}